local mod = EPGP:NewModule("EPGP_Export")

local L = LibStub:GetLibrary("AceLocale-3.0"):GetLocale("EPGP")

local json_writer = {}

local xtable = {
  ['"'] = '\\"',
  ['\\'] = '\\\\',
  ['/'] = '\\/',
  ['\b'] = '\\b',
  ['\f'] = '\\f',
  ['\n'] = '\\n',
  ['\r'] = '\\r',
  ['\t'] = '\\t',
}

function json_writer:string(output, val)
  val = string.gsub(val, '["\\/\b\f\n\r\t]', xtable)
  output('"')
  output(val)
  output('"')
end

function json_writer:number(output, val)
  output(tostring(val))
end

function json_writer:table(output, val)
  -- This check is not 100% safe, but it will do for this application.
  if val[1] then
    output("[")
    for i,v in ipairs(val) do
      if i ~= 1 then
        output(", ")
      end
      self:value(output, v)
    end
    output("]")
  else
    output("{")
    local first = true
    for k,v in pairs(val) do
      if first then
        first = false
      else
        output(", ")
      end
      self:value(output, k)
      output(": ")
      self:value(output, v)
    end
    output("}")
  end
end

function json_writer:boolean(output, val)
  output(tostring(val))
end

function json_writer:value(output, val)
  self[type(val)](self, output, val)
end

function toJSON(val)
  local t = {}
  local function output(str)
    table.insert(t, str)
  end
  json_writer:value(output, val)
  return table.concat(t)
end

function mod:Export(v)
  return toJSON(v)
end
